@@ -15,6 +15,14 @@ class TestSettingsInversion:
         settings = aa.SettingsInversion(use_linear_operators=False)
         assert settings.use_linear_operators_tag == "mat"
 
+    def test__inversion_use_preconditioner_tag(self):
+
+        settings = aa.SettingsInversion(use_preconditioner=True)
+        assert settings.use_preconditioner_tag == "__precon"
+
+        settings = aa.SettingsInversion(use_preconditioner=False)
+        assert settings.use_preconditioner_tag == ""
+
     def test__tag(self):
 
         settings = aa.SettingsInversion(use_linear_operators=True)
@@ -23,6 +31,11 @@ class TestSettingsInversion:
         settings = aa.SettingsInversion(use_linear_operators=False)
         assert settings.tag == "inv[mat]"
 
+        settings = aa.SettingsInversion(
+            use_linear_operators=False, use_preconditioner=True
+        )
+        assert settings.tag == "inv[mat__precon]"
+
 
 class TestLogDetMatrixCholesky:
     def test__determinant_of_positive_definite_matrix_via_cholesky(self):
